(* Allows more print depth.                                         *)
Compiler.Control.Print.printDepth:= 100;
    
signature ENGLISHLEXER =
sig
    val lex : string -> string list
    val lexstr : string -> string list
end


structure EnglishLexer: ENGLISHLEXER =
struct
    open TextIO;

    (* Return pair of sublists of 2nd argument occurring before and after
       first occurrence of c in 2nd argument.  
       accUpTo = fn : ''a -> ''a list -> ''a list * ''a list    *)
    fun accUpToWhiteSpace [] = ([], [])
      | accUpToWhiteSpace (fst::rest) = 
		if Char.isSpace fst then ([], rest)
                           else let 
				   val (bef,aft) = accUpToWhiteSpace rest
                                in 
				   (fst::bef,aft)
                                end;

    (* Return pair of substrings of str occurring before and after first
       occurrence of c 
       upTo = fn : char -> char list -> string * char list    *)
    fun upToWhiteSpace str = let 
			val (bef, aft) = accUpToWhiteSpace str
                     in 
			(implode bef, aft)
		     end;

    (* Return the list of tokens found in the input.
       Parameter:  A character list to tokenize
    *)
    fun gettokens nil = []
      | gettokens (fst::rest) =                 (* Untagged text *)
          if Char.isSpace fst then
	    (* Recurse to skip white space *)
	     gettokens rest
	  else 
	     let 
	        val (token,others) = upToWhiteSpace (fst::rest)
	     in
	        token::(gettokens others)
	     end;

    (* Returns the list of tokens found in a string. *)
    fun lexstr s = gettokens (explode s)
                
    (* Returns the list of tokens found in a file.
       Parameter:  filename *)
    fun lex file = 
        let
	    val strm = openIn file;
            val filecontents = explode (input strm)
        in 
            (closeIn strm; gettokens filecontents)
        end
end;


(*  Now define the parsing part of the program, which takes a list of  *)
(*  tokens and determines whether is a legal English sentence.         *)

signature ENGLISHPARSER =
sig
    val parse : string list -> bool
end

(* structure EnglishParser : ENGLISHPARSER =
struct *)
	   
    (* Parses Article.  If first token is the article "a" or "the", returns a
       tuple containing a true and the remaining tokens.  Otherwise it return
       a tuple of a false and the remaining tokens.  
    *)
      (* Article -> a | the *)
    fun parseArticle (fst::tokens) = 
            if fst = "a" orelse fst = "the" then (true, tokens)
	                                    else (false, tokens)

    (* Parses Noun.  If first token is the noun "girl" or "dog", returns a
       tuple containing a true and the remaining tokens.  Otherwise it return
       a tuple of a false and the remaining tokens.  
    *)
       (* Noun -> girl | dog  *)
    fun parseNoun (fst::tokens) = ???
    
    (* Parses verb.  If first token is the verb "sees" or "pets", returns a
       tuple containing a true and the remaining tokens.  Otherwise it return
       a tuple of a false and the remaining tokens.  
    *)
       (*  Verb -> sees | pets  *)
    fun parseVerb (fst::tokens) = ???

    (* parses a NounPhrase.  If the first two tokens represent an article
       and a noun then return the tuple consisting of true and the remaining
       tokens.  Otherwise it returns false and the remaining tokens. *)
       (* NounPhrase -> Article Noun  *)
    fun parseNounPhrase tokens = ???

    (* parses a VerbPhrase.  If we recognize a verb and then a NounPhrase
       then return the tuple consisting of true and the remaining
       tokens.  Otherwise it returns false and the remaining tokens. *)
       (* VerbPhrase -> Verb NounPhrase  *)
    fun parseVerbPhrase tokens = ???
		   
    (* Parses sentence.  If an expression is found, returns a
       tuple containing a boolean indicating whether the tokens represent 
       a sentence, and the input following the expression.  
    *)
       (*  Sentence -> NounPhrase VerbPhrase  *)
    fun parseSentence tokens = let
            val (OK, rest) = parseNounPhrase tokens
	  in
	    if OK then parseVerbPhrase rest
	          else (false, rest)
          end;

    (* Return true if the tokens represent an English sentence. *)
  fun parse tokens =
      let 
        val (OK, others) = parseSentence tokens;
      in
        OK (* andalso others = [] *)
      end
(* end; *)

(* The final definition of the function puts the pieces together.   *)

(* Parse input from a file named str *)
fun parsefile str = parse (EnglishLexer.lex str);

(* parse from a string, str *)
fun parsestr str = parse (EnglishLexer.lexstr str);

